//-------------------------------------------------------------
// InferenceEngineFBShared
// June 2014 by Michael Soukup
//-------------------------------------------------------------


#include "inferenceengine.h"
#ifdef _OPENMP
#include <omp.h>
#endif

// TODO this can easily be added.
int InferenceEngineFBShared::forwardBeliefLog(FeatureGenerator* fGen, Model* model,
						DataSequenceRealtime* dataSequence, dVector* prob){
	throw HcrfNotImplemented("Forward belief log is not implemented");
	return -1;
}


double InferenceEngineFBShared::computePartition(FeatureGenerator* fGen, DataSequence* X,
                            					 Model* crf, int seqLabel,
                            					 bool bUseStatePerNodes)
{
	Beliefs bel;
	computeBeliefsLog(bel, fGen, X, crf, bUseStatePerNodes, bUseStatePerNodes);
	return bel.partition;
}

void InferenceEngineFBShared::computeBeliefs(Beliefs& bel, FeatureGenerator* fGen,DataSequence* X,
                        					 Model* crf, int bComputePartition, int seqLabel,
                        					 bool bUseStatePerNodes)
{
	// We might also add linear later
	computeBeliefsLog(bel, fGen, X, crf, bUseStatePerNodes);
}


void InferenceEngineFBShared::computeBeliefsLinear(Beliefs& bel, FeatureGenerator* fGen,
									  DataSequence* X, Model* model, bool bUseStatePerNodes)
{
	if(model->getAdjacencyMatType()!=CHAIN){
		throw HcrfBadModel("InferenceEngineFB need a model based on a Chain");
	}
	int NNODES=X->length();
	int NSTATES = model->getNumberOfStates();
	bel.belStates.resize(NNODES);
	for(int i=0;i<NNODES;i++)
	{
		bel.belStates[i].create(NSTATES);
		bel.belStates[i].set(0);
	}
	int NEDGES = NNODES-1;
	bel.belEdges.resize(NEDGES);
	for(int i=0;i<NEDGES;i++)
	{
		bel.belEdges[i].create(NSTATES,NSTATES);
		bel.belEdges[i].set(0);
	}
	dMatrix Mi_YY (NSTATES,NSTATES);
	dVector Ri_Y (NSTATES);
	dVector alpha_Y(NSTATES);
	dVector newAlpha_Y(NSTATES);
	dVector tmp_Y(NSTATES);
	alpha_Y.set(1);
	bel.belStates[NNODES-1].set(1.0);
	for (int i = NNODES-1; i > 0; i--)
	{
		// compute the Mi matrix
		computeLogMiShared(fGen,model,X,i,Mi_YY,Ri_Y,true,
					 bUseStatePerNodes);
		tmp_Y.set(bel.belStates[i]);
		tmp_Y.eltMpy(Ri_Y);
		bel.belStates[i-1].multiply(Mi_YY,tmp_Y);
	}
	for (int i = 0; i < NNODES; i++)
	{
		// compute the Mi matrix
		computeLogMiShared(fGen,model,X,i,Mi_YY,Ri_Y, true,
					 bUseStatePerNodes);

		if (i > 0)
		{
			tmp_Y.set(alpha_Y);
			Mi_YY.transpose();
			newAlpha_Y.multiply(Mi_YY,tmp_Y);
			newAlpha_Y.eltMpy(Ri_Y);
		}
		else
		{
			newAlpha_Y.set(Ri_Y);
		}

		if (i > 0)
		{
			tmp_Y.set(Ri_Y);
			tmp_Y.eltMpy(bel.belStates[i]);
			tmp_Y.transpose();
			bel.belEdges[i-1].multiply(alpha_Y,tmp_Y);
			Mi_YY.transpose();
			bel.belEdges[i-1].eltMpy(Mi_YY);
		}

		bel.belStates[i].eltMpy(newAlpha_Y);
		alpha_Y.set(newAlpha_Y);
	}
	double Zx = alpha_Y.sum();
	for (int i = 0; i < NNODES; i++)
	{
		bel.belStates[i].multiply(1.0/Zx);
	}
	for (int i = 0; i < NEDGES; i++)
	{
		bel.belEdges[i].multiply(1.0/Zx);
	}
	bel.partition = log(Zx);
}


// Computes p(h_t|xi,yi) and p(h_t-1,h_t|xi,yi)
void InferenceEngineFBShared::computeBeliefsLog(Beliefs& bel, FeatureGenerator* fGen,
										 		DataSequence* X, Model* model,
												bool bUseStatePerNodes, bool bObj)
{
	if(model->getAdjacencyMatType()!=CHAIN){
		throw HcrfBadModel("InferenceEngineFB need a model based on a Chain");
	}
	int NNODES = X->length();
	int NSTATES = model->getNumberOfStates();
	int NLABELS = model->getNumberOfStateLabels();
	bel.belStates.resize(NNODES);

	for(int i=0;i<NNODES;i++)
	{
		bel.belStates[i].create(NSTATES);
	}

	int NEDGES = NNODES-1;
	bel.belEdges.resize(NEDGES);
	for(int i=0;i<NEDGES;i++)
	{
		bel.belEdges[i].create(NSTATES,NSTATES, 0);
	}
	dMatrix Mi_YY (NSTATES,NSTATES);
	dVector Ri_Y (NSTATES);
	dVector alpha_Y(NSTATES);
	dVector newAlpha_Y(NSTATES);
	dVector tmp_Y(NSTATES);

	// compute beta values in a backward scan.
	// also scale beta-values to 1 to avoid numerical problems.
	alpha_Y.set(0);
	bel.belStates[NNODES-1].set(0);
	for (int i = NNODES-1; i > 0; i--)
	{
		// compute the Mi matrix
		computeLogMiShared(fGen, model, X, i, Mi_YY, Ri_Y, false, bUseStatePerNodes, bObj);
		tmp_Y.set(bel.belStates[i]);
		tmp_Y.add(Ri_Y);
		LogMultiply(Mi_YY,tmp_Y,bel.belStates[i-1]);
	}

	// Compute Alpha values
	for (int i = 0; i < NNODES; i++) {
		// compute the Mi matrix
		computeLogMiShared(fGen,model, X, i, Mi_YY, Ri_Y, false, bUseStatePerNodes, bObj);
		if (i > 0)
		{
			tmp_Y.set(alpha_Y);
			Mi_YY.transpose();
			LogMultiply(Mi_YY, tmp_Y, newAlpha_Y);
			newAlpha_Y.add(Ri_Y);
		}
		else
		{
			newAlpha_Y.set(Ri_Y);
		}
		if (i > 0)
		{
			tmp_Y.set(Ri_Y);
			tmp_Y.add(bel.belStates[i]);
			Mi_YY.transpose();
			bel.belEdges[i-1].set(Mi_YY);
			for(int yprev = 0; yprev < NSTATES; yprev++)
				for(int yp = 0; yp < NSTATES; yp++)
					bel.belEdges[i-1](yprev,yp) += tmp_Y[yp] + alpha_Y[yprev];
		}
	  
		bel.belStates[i].add(newAlpha_Y);
		alpha_Y.set(newAlpha_Y);
	}

	double lZx = alpha_Y.logSumExp();
	for (int i = 0; i < NNODES; i++)
	{
		bel.belStates[i].add(-lZx);
		bel.belStates[i].eltExp();
	}
	for (int i = 0; i < NEDGES; i++)
	{
		bel.belEdges[i].add(-lZx);
		bel.belEdges[i].eltExp();
	}
	bel.partition = lZx;
}


void InferenceEngineFBShared::computeLogMiShared(FeatureGenerator* fGen, Model* model,
								   		   DataSequence* X, int i,
								  		   dMatrix& Mi_YY, dVector& Ri_Y,
								  		   bool takeExp, bool bUseStatePerNodes, bool bObj)
{
	// This function compute Ri_Y = Sum of features for every states at time i
	// and Mi_YY, the sum of feature for every transition.
	Mi_YY.set(0);
	Ri_Y.set(0);
	dVector* lambda=model->getWeights(-1);

#if defined(_OPENMP)
	int ThreadID = omp_get_thread_num();
	if (ThreadID >= nbThreadsMP)
		ThreadID = 0;
#else
	int ThreadID = 0;
#endif

	// Window Raw Features
#if defined(_VEC_FEATURES) || defined(_OPENMP)
	fGen->getFeatures(vecFeaturesMP[ThreadID], X, model, i, -1, -1);
	feature* pFeature = vecFeaturesMP[ThreadID].getPtr();
	for (int j = 0; j < vecFeaturesMP[ThreadID].size() ; j++, pFeature++)
#else
	featureVector* vecFeature = fGen->getFeatures(X,model, i, -1 ,-1);
	feature* pFeature = vecFeature->getPtr();
	for (int j = 0; j < vecFeature->size() ; j++, pFeature++)
#endif
	{
		double oldVal = Ri_Y.getValue(pFeature->nodeState);
		Ri_Y.setValue(pFeature->nodeState,oldVal+(*lambda)[pFeature->globalId]*pFeature->value);
	}


	int s = X->getStateLabels(i);
	dVector Gi_Y(model->getNumberOfStates());
	Gi_Y.set(0);
	// Latent Label (shared) Features
#if defined(_VEC_FEATURES) || defined(_OPENMP)
	fGen->getFeatures(vecFeaturesMP[ThreadID], X, model, i, i+X->length(), -1);
	pFeature = vecFeaturesMP[ThreadID].getPtr();
	for (int j = 0; j < vecFeaturesMP[ThreadID].size() ; j++, pFeature++)
#else
	vecFeature = fGen->getFeatures(X,model, i, i+X->length() , -1);
	pFeature = vecFeature->getPtr();
	for (int j = 0; j < vecFeature->size() ; j++, pFeature++)
#endif
	{
		// If bUseStatePerNodes, we mask the beliefs for this y. Only used during training.
		// If not bUseStatePerNodes we drop gk entirely.
		//if ((bUseStatePerNodes && pFeature->prevNodeState ==s) || bObj) {
		if ((bUseStatePerNodes && pFeature->prevNodeState ==s) || bObj) {
			double oldVal = Gi_Y.getValue(pFeature->nodeState);
			Gi_Y.setValue(pFeature->nodeState, oldVal+(*lambda)[pFeature->globalId]*pFeature->value);
		}
	}

	double maskValue = -INF_VALUE;
	//for (int h = 0; h<model->getNumberOfStates(); h++) {
	//	if (bUseStatePerNodes && Gi_Y.getValue(h) < 0) {
	//		Gi_Y.setValue(h, -1);
	//	}
	//}

	Ri_Y.add(Gi_Y);

	// Edge Features
	if(i>0 ) {
#if defined(_VEC_FEATURES) || defined(_OPENMP)
		fGen->getFeatures(vecFeaturesMP[ThreadID], X, model, i, i-1, -1);
		pFeature = vecFeaturesMP[ThreadID].getPtr();
		for (int j = 0; j < vecFeaturesMP[ThreadID].size() ; j++, pFeature++) {
#else
		vecFeature = fGen->getFeatures(X, model, i, i-1, -1);
		pFeature = vecFeature->getPtr();
		for (int j = 0; j < vecFeature->size() ; j++, pFeature++) {
#endif
			int f = pFeature->id;
			int yp = pFeature->nodeState;
			int yprev = pFeature->prevNodeState;
			double val = pFeature->value;
			Mi_YY.setValue(yprev, yp, Mi_YY.getValue(yprev,yp)+(*lambda)[f]*val);
		}
	}

	if (takeExp) {
		Ri_Y.eltExp();
		Mi_YY.eltExp();
		maskValue = 0;
	}
}

