#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame, time, random
from errp_path import PATHS


KEYMAP = {
	pygame.K_UP: (0,-1),
	pygame.K_DOWN: (0,1),
	pygame.K_LEFT: (-1,0),
	pygame.K_RIGHT: (1,0)
}
SCREEN_DIM = (800, 600)
R = 5
DURATION = 60*5 # Limit to 5 min
COOLDOWN = 5
T_CHCMD = 10
P_CHCMD = 0.3

pygame.init()
screen = pygame.display.set_mode(SCREEN_DIM)
pygame.display.set_caption("ErrP experiment - follow path")
font = pygame.font.SysFont(None, 24)
clock = pygame.time.Clock()

def _change_cmd(cmd):
    new_cmd = cmd
    while new_cmd == cmd:
        new_cmd = random.choice(KEYMAP.values())
    return new_cmd

def _disptxt(s_arr, col):
	global screen, font
	screen.fill((255, 255, 255))
	for i,s in enumerate(s_arr):
		screen.blit(font.render(s, True, col), (10, 10 + i*20))
	pygame.display.flip()


_disptxt(["Get ready.."], (0,0,0))
time.sleep(1)
_disptxt(["3"], (0,0,0))
time.sleep(1)
_disptxt(["2"], (0,0,0))
time.sleep(1)
_disptxt(["1"], (0,0,0))
time.sleep(1)

rpath = random.choice(PATHS)
path = []
pos = rpath[0]
prev_pos = pos
vel = 10
errps = []
t_start = time.time()
t_elapsed = 0
t_lastch = 0
prev_cmd = (0,0)
chcmd = False

running = True
while t_elapsed < DURATION and running:
    pressed = pygame.key.get_pressed()
    for e in pygame.event.get():
        if e.type == pygame.QUIT or pressed[pygame.K_ESCAPE]:
            running = False

    cmd = (0,0)
    for k in KEYMAP.keys():
        if pressed[k]:
            cmd = KEYMAP[k]
    if chcmd:
        if cmd == (0,0):
            chcmd = False
        else:
            cmd = prev_cmd
    elif cmd != (0,0) and prev_cmd != cmd \
         and t_elapsed - t_lastch > COOLDOWN \
         and t_elapsed > T_CHCMD and random.random() < P_CHCMD:
        cmd = _change_cmd(cmd)
        t_lastch = t_elapsed
        chcmd = True
        errps.append(t_elapsed)
    else:
        chcmd = False
    
    x, y = map(sum, zip(pos, cmd))
    x = max(0, min(x, SCREEN_DIM[0]))
    y = max(0, min(y, SCREEN_DIM[1]))
    pos = (x, y)
    if pos != prev_pos:
        path.append(pos)

    screen.fill((255,255,255))
    for p in rpath:
        pygame.draw.circle(screen, (150,150,150), map(int, p), R-2)
    for p in path:
        pygame.draw.circle(screen, (50,50,50), map(int, p), R-2)
    pygame.draw.circle(screen, (255,0,0), map(int, pos), R)
    pygame.display.flip()

    prev_pos = pos
    prev_cmd = cmd
    t_elapsed = time.time() - t_start
    clock.tick(60)

_disptxt(["Cleaning up..."], (0,0,0))
pygame.quit()

