#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame, sys, time, random, threading
from emotiv import epoc, utils
from errp_path import PATHS

try:
    filename = sys.argv[1]
except Exception:
    print "Usage: %s [mat-file]" % sys.argv[0]
    sys.exit(1)


KEYMAP = {
	pygame.K_UP: (0,-1),
	pygame.K_DOWN: (0,1),
	pygame.K_LEFT: (-1,0),
	pygame.K_RIGHT: (1,0)
}
SCREEN_DIM = (800, 600)
R = 5
DURATION = 60
COOLDOWN = 5
T_CHCMD = 10
P_CHCMD = 0.3

pygame.init()
screen = pygame.display.set_mode(SCREEN_DIM)
pygame.display.set_caption("ErrP experiment - follow path")
font = pygame.font.SysFont(None, 24)
clock = pygame.time.Clock()
headset = None
eegrec = []
t_start_th = None

class EEGRecord(threading.Thread):
    def run(self):
        global headset, eegrec, t_start_th, DURATION
        print "starting thread"
        t_start_th = time.time()
        idx, eegrec = headset.acquire_data_fast(DURATION)
        print "Thread fin"

def _change_cmd(cmd):
    new_cmd = cmd
    while new_cmd == cmd:
        new_cmd = random.choice(KEYMAP.values())
    return new_cmd

def _disptxt(s_arr, col):
	global screen, font
	screen.fill((255, 255, 255))
	for i,s in enumerate(s_arr):
		screen.blit(font.render(s, True, col), (10, 10 + i*20))
	pygame.display.flip()


_disptxt(["Initializing headset.."], (0,0,0))
try:
    headset = epoc.EPOC(enable_gyro=False)
    _disptxt(["Headset initialized."], (0,0,0))
except Exception:
    _disptxt(["Headset initiation failed.", "Running without headset..."], (0,0,0))
time.sleep(2)

_disptxt(["Get ready.."], (0,0,0))
time.sleep(1)
_disptxt(["3"], (0,0,0))
time.sleep(1)
_disptxt(["2"], (0,0,0))
time.sleep(1)
_disptxt(["1"], (0,0,0))
time.sleep(1)

if headset:
    eegrec_th = EEGRecord()
    eegrec_th.start()

rpath = random.choice(PATHS)
path = []
pos = rpath[0]
prev_pos = pos
vel = 10
errps = []
noerrps = []
t_start = time.time()
t_elapsed = 0
t_lastch = 0
prev_cmd = (0,0)
chcmd = False
running = True
while t_elapsed < DURATION and running:
    pressed = pygame.key.get_pressed()
    for e in pygame.event.get():
        if e.type == pygame.QUIT or pressed[pygame.K_ESCAPE]:
            running = False

    cmd = (0,0)
    for k in KEYMAP.keys():
        if pressed[k]:
            cmd = KEYMAP[k]
    if chcmd:
        if cmd == (0,0):
            chcmd = False
        else:
            cmd = prev_cmd
    elif cmd != (0,0) and prev_cmd != cmd:
        # new command
        if t_elapsed - t_lastch > COOLDOWN and t_elapsed > T_CHCMD and random.random() < P_CHCMD:
            # Produce error
            cmd = _change_cmd(cmd)
            t_lastch = t_elapsed
            chcmd = True
            errps.append(t_elapsed)
        else:
            noerrps.append(t_elapsed)
    else:
        chcmd = False
    
    x, y = map(sum, zip(pos, cmd))
    x = max(0, min(x, SCREEN_DIM[0]))
    y = max(0, min(y, SCREEN_DIM[1]))
    pos = (x, y)
    if pos != prev_pos:
        path.append(pos)

    screen.fill((255,255,255))
    for p in rpath:
        pygame.draw.circle(screen, (150,150,150), map(int, p), R-2)
    for p in path:
        pygame.draw.circle(screen, (50,50,50), map(int, p), R-2)
    pygame.draw.circle(screen, (255,0,0), map(int, pos), R)
    pygame.display.flip()

    prev_pos = pos
    prev_cmd = cmd
    t_elapsed = time.time() - t_start
    clock.tick(60)

if not headset:
    _disptxt(["Cleaning up..."], (0,0,0))
    pygame.quit()
    sys.exit(0)

_disptxt(["Waiting for recording..."], (0,0,0))
while eegrec_th.is_alive():
    time.sleep(0.1)
    
_disptxt(["Saving mat-file..."], (0,0,0))
t_diff = t_start - t_start_th
for e in errps:
    e += t_diff
for ne in noerrps:
    ne += t_diff

if len(eegrec) > 0:
    utils.save_as_matlab(eegrec, headset.channel_mask,
                         filename=filename,
                         metadata=dict(errps=errps, noerrps=noerrps))

_disptxt(["Cleaning up..."], (0,0,0))
headset.disconnect()
pygame.quit()

print "ErrP t:"
print errps
print "eegrec len:"
print len(eegrec)

