import urx, math

def normalize(vec):
	dim = len(vec)
	l = math.sqrt(sum([math.pow(i, dim) for i in vec]))
	return [i/l for i in vec]


class RobotInitException(Exception):
	pass


class UR5Controller(object):
	"""Implements continuous linear control for the UR5 robot."""

	def __init__(self, ip):
		try:
			self.robot = urx.Robot(ip, useRTInterface=True)
		except Exception, e:
			raise RobotInitException('Robot initiation failed: %s' % str(e))

		# TODO tmp
		#self.vel = 0.1
		#self.acc = 1.0
		#self.stop_acc = 0.3

		self.mode = 'cartesian'
		
		self.axis_x = (1,0,0)
		self.axis_y = (0,1,0)
		self.axis_z = (0,0,1)

		self.pos = self.robot.getl()
		self.vel = [0]*6
		self.acc = [0]*6
		self.prev_ui_vec = [0]*6 # TODO tmp


	def setjoints(self, joints):
		self.robot.movej(joints, acc=0.1, vel=0.5, radius=0, wait=True, relative=False)

	def set_control_mode(self, mode):
		self.mode = mode

	def calibrate_csys(self):
		theta_d = 0.02
		j1 = self.robot.getj()
		j2 = [theta+theta_d if i==1 else theta for i,theta in enumerate(j1)]
		p1 = self.robot.getl()

		# Make a small move with joint 1 outwards and inwards
		self.robot.movej(j2, acc=0.1, vel=0.5, radius=0, wait=True)
		p2 = self.robot.getl()
		self.robot.movej(j1, acc=0.1, vel=0.5, radius=0, wait=True)

		self.axis_y = tuple(normalize((p1[0]-p2[0], p1[1]-p2[1], 0)))
		self.axis_x = (self.axis_y[1], -self.axis_y[0], 0) # Rotate 90 deg

	def get_tcp_force(self):
		return self.robot.get_tcp_force(wait=False)

	def _translate_vec(self, vec):
		# change of basis to calibrated axes
		x, y, z = vec[0:3]
		v = [self.axis_x[i]*x + self.axis_y[i]*y + self.axis_z[i]*z for i in range(3)]
		return v


	def follow_trajectory(self, ui_vec, dt):
		if self.prev_ui_vec == ui_vec:
			return
		# new command
		self.prev_ui_vec = ui_vec

		if sum(map(abs, ui_vec)) > 0:
			self.pos = self.robot.getl()
			move_vec = self._translate_vec(ui_vec)
			move_vec[3::] = self.pos[3::]
			self.robot.movel(move_vec, acc=0.8, vel=0.1, radius=0, wait=False)
		else:
			self.robot.stopl(1.0)


	def move(self, ui_vec, dt):

		# Update position, velocity and acceleration vectors
		new_pos = self.robot.getl() if self.mode == 'cartesian' else self.robot.getj()
		new_vel = [dp/dt for dp in map(sum, zip(new_pos, [-i for i in self.pos]))]
		self.acc = [dv/dt for dv in map(sum, zip(new_vel, [-i for i in self.vel]))]
		self.pos = new_pos
		self.vel = new_vel

		# TODO check bounds
		
		if self.prev_ui_vec == ui_vec: # tmp
			return
		self.prev_ui_vec = ui_vec

		if sum(map(abs, ui_vec)) > 0:
			if self.mode == 'cartesian':
				# translate axes
				move_vec = self._translate_vec(ui_vec)
				move_vec[3::] = self.pos[3::]
				self.robot.movel(move_vec, acc=0.8, vel=0.1, radius=0, wait=False)

			else:
				joints = map(sum, zip(self.pos, ui_vec))
				self.robot.movej(joints, acc=0.8, vel=0.1, radius=0, wait=False)
		else:
			self.robot.stopl(0.3)


		# poll input, calculate move vector
		# update states


	def cleanup(self):
		self.robot.cleanup()

