% - Loads subject data from BCI competition III, data set V
% - Slice the data into training instances with length T seconds;
% - Computes average bandpower in alpha (8-12Hz), sigma (12-16Hz) and beta (18-26Hz) bands
% - Selects a subset of features for each subject
% - ...



DPATH = '../../data/bcicompetition/datasetv/%s_subject%d_psd0%d.mat';
N_SUBJ = 3;                 % Number of subjects
N_TRIAL = 3;                % Number of trials per subject
N_CHAN = 8;                 % C3, Cz, C4, CP1, CP2, P3, Pz, and P4.
N_FREQS = 12;               % 8-30Hz: 8, 10, 12 ... 30
FDIM = N_CHAN*N_FREQS;

fs = 16;                    % Sample frequency
T = 10;                     % Length of each sequence. The data is sliced accordingly.
Ns = T*fs;

subjectData = {};           % Container that holds data for all subjects

for sid=1:N_SUBJ
    
    % produce features and merge trials
    x = [];
    y = [];
    for tid=1:N_TRIAL
        f = sprintf(DPATH, 'train', sid, tid);
        load(f);
        
        Nmin = min(size(X, 1), size(Y, 1)); % Slice of to equal length
        x = [x X(1:Nmin, :)'];
        y = [y Y(1:Nmin, :)'];
    end
    
    features = featuresAvrgBP(x);
    featureSubset = featureSelectionLDCRF(features, sid);
    y = fixLabels(y);
    
    % slice into sequences of length Ns
    Ntot = size(featureSubset, 2);
    Nseqs = ceil(Ntot / Ns);
    subjectData{sid}.trainSeqs = cell(1,Nseqs);
    subjectData{sid}.trainLabels = cell(1,Nseqs);
    
    offset = 0;
    for i=1:Nseqs
        if (offset + Ns) <= Ntot
            subjectData{sid}.trainSeqs{i} = featureSubset(:, (offset+1):(offset+Ns));
            subjectData{sid}.trainLabels{i} = y(:, (offset+1):(offset+Ns));
        else
            subjectData{sid}.trainSeqs{i} = featureSubset(:, (offset+1):end);
            subjectData{sid}.trainLabels{i} = y(:, (offset+1):end);
        end
        offset = offset + Ns;
    end
    
    
    % load test data
    f = sprintf(DPATH, 'test', sid, 4);
    load(f);
    
    Nmin = min(size(X, 1), size(Y, 1)); % Slice of to equal length
   
    features = featuresAvrgBP(X(1:Nmin, :)');
    featureSubset = featureSelectionLDCRF(features, sid);
    y = fixLabels(Y(1:Nmin, :)');
    
    % we dont slice the test data
    subjectData{sid}.testSeqs = cell(1,1);
    subjectData{sid}.testLabels = cell(1,1);
    subjectData{sid}.testSeqs{1} = featureSubset;
    subjectData{sid}.testLabels{1} = y;
    
end



